import { none, isNone } from "@reactive-js/option";
import { useObservable, useAsyncEnumerable } from "@reactive-js/react";
import { map, scan, never } from "@reactive-js/observable";
import { pipe } from "@reactive-js/pipe";
import { createElement, useMemo } from "react";
const empty = {
    path: "",
    query: "",
    fragment: "",
};
const pairify = ([_, oldState], next) => oldState === empty ? [none, next] : [oldState, next];
export const Router = function Router(props) {
    const { location, notFound, routes, scheduler } = props;
    const locationStore = useAsyncEnumerable(location, { replay: 1 });
    const observable = useMemo(() => {
        if (isNone(locationStore)) {
            return never();
        }
        else {
            const routeMap = {};
            for (const [path, component] of routes) {
                routeMap[path] = component;
            }
            const uriUpdater = (updater) => {
                locationStore.dispatch(updater);
            };
            return pipe(locationStore, scan(pairify, () => [none, empty]), map(([referer, uri]) => {
                var _a;
                return createElement((_a = routeMap[uri.path]) !== null && _a !== void 0 ? _a : notFound, {
                    referer,
                    uri,
                    uriUpdater,
                });
            }));
        }
    }, [locationStore, notFound, routes]);
    const element = useObservable(observable, scheduler);
    return element !== null && element !== void 0 ? element : null;
};
//# sourceMappingURL=router.js.map