import { useCallback, useMemo } from "react";
const createURIStateUpdater = (stateUpdater, parse, serialize, stateIsQuery) => (oldURI) => {
    const oldSerialized = stateIsQuery ? oldURI.query : oldURI.fragment;
    const oldState = parse(oldSerialized);
    const newState = stateUpdater(oldState);
    if (oldState === newState) {
        return oldURI;
    }
    else if (stateIsQuery) {
        const query = serialize(newState);
        return Object.assign(Object.assign({}, oldURI), { query });
    }
    else {
        const fragment = serialize(newState);
        return Object.assign(Object.assign({}, oldURI), { fragment });
    }
};
export const useRoutableState = (props, parse, serialize, stateIsQuery = false) => {
    const { uri: { query, fragment }, uriUpdater, } = props;
    const state = useMemo(() => {
        const serialized = stateIsQuery ? query : fragment;
        return parse(serialized);
    }, [parse, query, fragment]);
    const notify = useCallback((stateUpdater) => {
        uriUpdater(createURIStateUpdater(stateUpdater, parse, serialize, stateIsQuery));
    }, [uriUpdater, parse, serialize, stateIsQuery]);
    return [state, notify];
};
//# sourceMappingURL=useRoutableState.js.map